@@ -558,6 +558,103 @@ And the second sentence.
         ret = texplain.indent(text)
         self.assertEqual(ret.strip(), formatted.strip())
 
+    def test_newline(self):
+        text = r"""
+This is a \\
+long sentence.
+
+With some
+more words.
+        """
+
+        formatted = r"""
+This is a \\
+long sentence.
+
+With some more words.
+        """
+
+        ret = texplain.indent(text)
+        self.assertEqual(ret.strip(), formatted.strip())
+
+    def test_newline_b(self):
+        text = r"""
+This is a \\
+long sentence.
+
+With some \\ more. And some more words.
+
+\\ Here is
+another
+bit of
+text.
+        """
+
+        formatted = r"""
+This is a \\
+long sentence.
+
+With some \\ more.
+And some more words.
+
+\\ Here is another bit of text.
+        """
+
+        ret = texplain.indent(text)
+        self.assertEqual(ret.strip(), formatted.strip())
+
+    def test_newline_c(self):
+        text = r"""
+This is a \\ \\
+long sentence.
+
+With some \\ \\ more. And some more words.
+
+
+\\ \\ Here is
+another
+bit of
+text.
+        """
+
+        formatted = r"""
+This is a \\ \\
+long sentence.
+
+With some \\ \\ more.
+And some more words.
+
+\\ \\ Here is another bit of text.
+        """
+
+        ret = texplain.indent(text)
+        self.assertEqual(ret.strip(), formatted.strip())
+
+    def test_multiline(self):
+        text = r"""
+This is a \\
+long sentence.
+
+With
+some
+more
+words.
+And
+another
+following sentence.
+        """
+
+        formatted = r"""
+This is a \\
+long sentence.
+
+With some more words.
+And another following sentence.
+        """
+
+        ret = texplain.indent(text)
+        self.assertEqual(ret.strip(), formatted.strip())
+
     def test_environment(self):
         text = r"""
 This is the
