@@ -12,16 +12,39 @@ from shutil import copyfile
 import numpy as np
 from numpy.typing import ArrayLike
 from numpy.typing import NDArray
+from typing_extensions import deprecated
 
 from ._version import version  # noqa: F401
 from ._version import version_tuple  # noqa: F401
 
 
 class PlaceholderType(enum.Enum):
-    """
+    r"""
     Type of placeholder.
+    The placeholders' practical definition is in :py:func:`text_to_placeholder`.
+    The intended use is:
+
+    :py:attr:`line`: A single line of content (1 line).
+    :py:attr:`inline_comment`: A comment that is preceded by some content.
+    :py:attr:`comment`: A comment that is the only content on that line.
+    :py:attr:`tabular`: Entire block ``\begin{tabular} ... \end{tabular}``.
+    :py:attr:`math`: Entire block of displaystyle math. E.g. ``\begin{equation} ... \end{equation}``
+    :py:attr:`inline_math`: Entire block of inline math. E.g. ``$ ... $``.
+    :py:attr:`math_line`: A single line of content in math mode.
+    :py:attr:`environment`: Entire block of environment: ``\begin{...} ... \end{...}``.
+    :py:attr:`command`: Entire block of command: ``\command[...]{...}``.
+    :py:attr:`curly_braced`: Entire block of curly braced content: ``{...}``.
+    :py:attr:`command_like`: Entire block of :py:attr:`command` or :py:attr:`curly_braced`.
+    :py:attr:`noindent_block`: Entire block of ``% \begin{noindent} ... % \end{noindent}``.
+    :py:attr:`verbatim`: Entire block of ``\begin{verbatim} ... \end{verbatim}``.
+    :py:attr:`let_command`: Definition ``\let...``.
+    :py:attr:`newif_command`: Definition ``\newif...``.
+
+    Except for :py:attr:`line`, :py:attr:`math_line`, :py:attr:`comment`, :py:attr:`inline_comment`
+    all placeholders can span more than one line.
     """
 
+    line = enum.auto()
     inline_comment = enum.auto()
     comment = enum.auto()
     tabular = enum.auto()
@@ -31,12 +54,14 @@ class PlaceholderType(enum.Enum):
     environment = enum.auto()
     command = enum.auto()
     curly_braced = enum.auto()
+    command_like = enum.auto()
     noindent_block = enum.auto()
     verbatim = enum.auto()
-    let = enum.auto()
-    newif = enum.auto()
+    let_command = enum.auto()
+    newif_command = enum.auto()
 
 
+@deprecated("Use regex directly")
 def find_opening(
     text: str,
     opening: str,
@@ -65,14 +90,14 @@ def find_commented(text: str) -> list[list[int]]:
     The output is such that one can find the comments text as follows::
 
         for i, j in find_commented(text):
-            print(text[i + 1 : j]) # i is the index of "%"
+            print(text[i : j]) # i is the index of "%"
 
-    :param text: The string to consider.
+    :param text: Text.
     :return: List of of indices of the beginning and end of the comments.
     """
 
-    comments = np.array(find_opening(text, "%", ignore_escaped=True))
-    newlines = np.array(find_opening(text, "\n", ignore_escaped=False) + [len(text)])
+    comments = np.array([i.span()[0] for i in re.finditer(r"(?<!\\)(%)", text)])
+    newlines = np.array([i.span()[0] for i in re.finditer(r"\n", text)] + [len(text)])
 
     ret = []
 
@@ -88,8 +113,8 @@ def is_commented(text: str) -> NDArray[np.bool_]:
     """
     Return array that lists per character if it corresponds to commented text.
 
-    :param text: The string to consider.
-    :return: Array of booleans.
+    :param text: Text.
+    :return: Array of booleans of size ``len(text)``.
     """
 
     comments = find_commented(text)
@@ -105,7 +130,8 @@ def find_matching_index(
     return_array: bool = False,
 ) -> dict:
     r"""
-    Find matching 'brackets'.
+    Find matching 'brackets', based on a list of indices corresponding to opening and closing
+    'brackets'.
 
     :param opening: Indices of the opening brackets.
     :param closing: Indices of the closing brackets.
@@ -216,6 +242,9 @@ def _detail_find_option(
 
     :param ret:
         List of tuples of the indices of the opening and closing braces.
+
+    :return:
+        Updated ``ret``.
     """
 
     if len(braces) == 0:
@@ -271,6 +300,7 @@ def _find_option(
     return _detail_find_option(character, index, braces, [])
 
 
+# TODO: This function should be able to be limited to a given number of options and arguments.
 def find_command(
     text: str,
     name: str = None,
@@ -329,7 +359,8 @@ def find_command(
     if curly_closing.size > 0:
         curly_closing = curly_closing[~is_comment[curly_closing]]
 
-    # search to what commands brackets might belong
+    # search to which command brackets might belong
+    # this is a rough estimate as commands might be nested
     cmd_square_open = np.searchsorted(cmd_end, square_open, side="right") - 1
     cmd_square_closing = np.searchsorted(cmd_end, square_closing, side="right") - 1
     cmd_curly_open = np.searchsorted(cmd_end, curly_open, side="right") - 1
@@ -379,8 +410,8 @@ def remove_comments(text: str) -> str:
     """
     Remove comments from a string.
 
-    :param text: The string to consider.
-    :return: The string without comments.
+    :param text: Text
+    :return: Text without comments.
     """
     text = text.splitlines()
     for i in range(len(text)):
@@ -479,30 +510,39 @@ class Placeholder:
             text[:start] + placeholder + text[end:],
         )
 
-    def to_text(self, text: str, index: int = None) -> str:
+    def to_text(self, text: str, index: int = None, keep_placeholder: bool = False) -> str:
         """
         Replace placeholder with content.
         If the whitespace before and after the placeholder is stored, it is restored.
 
-        :param text: The string to consider.
+        :param text: Text.
         :param index: The index of the placeholder.
+        :param keep_placeholder: If ``True`` the placeholder is kept (it is merely positioned).
+        :return: Text with placeholder replaced by content.
         """
+
         if index is None:
             index = text.find(self.placeholder)
 
-        pre = text[:index][::-1]
+        # placeholder not found
+        if index == -1:
+            return text
+
+        pre = text[:index]
         post = text[index + len(self.placeholder) :]  # noqa: E203
 
         if self.space_front is not None:
+            pre = pre[::-1]
             front = re.search(r"\s*", pre).end()
             pre = pre[front:][::-1] + self.space_front
-        else:
-            pre = pre[::-1]
 
         if self.space_back is not None:
             back = re.search(r"\ *\n?", post).end()
             post = self.space_back + post[back:]
 
+        if keep_placeholder:
+            return pre + self.placeholder + post
+
         return pre + self.content + post
 
     def __repr__(self) -> str:
@@ -546,6 +586,13 @@ class GeneratePlaceholder:
 
 
 def _filter_nested(indices: ArrayLike) -> ArrayLike:
+    """
+    Filter nested indices.
+
+    :param indices: A list of start and end indices.
+    :return: The filtered list of start and end indices.
+    """
+
     indices = indices[np.argsort(indices[:, 0])]
     keep = np.ones(len(indices), dtype=bool)
 
@@ -736,7 +783,7 @@ def _detail_text_to_placholders(
 
         return text, ret
 
-    if ptype == PlaceholderType.command:
+    if ptype == PlaceholderType.command or ptype == PlaceholderType.command_like:
         if placeholders_comments is not None:
             is_comment = _is_placeholder(text, placeholders_comments)
             components = find_command(text, is_comment=is_comment)
@@ -749,15 +796,18 @@ def _detail_text_to_placholders(
                 continue
             indices += [[component[0][0], component[-1][1]]]
 
+        if ptype == PlaceholderType.command_like:
+            indices += find_matching(text, "{", "}", ignore_escaped=True, closing_match=1).items()
+
         return _apply_placeholders(text, indices, base, "command".upper(), ptype)
 
     if ptype == PlaceholderType.curly_braced:
         indices = find_matching(
             text, "{", "}", ignore_escaped=True, closing_match=1, return_array=True
         )
-        return _apply_placeholders(text, indices, base, "nested".upper(), ptype)
+        return _apply_placeholders(text, indices, base, "curly-braced".upper(), ptype)
 
-    if ptype == PlaceholderType.let:
+    if ptype == PlaceholderType.let_command:
         regex = r"(?<!\\)(\\let)((\\[\w\@\*]*))*"
         if placeholders_comments is not None:
             is_comment = _is_placeholder(text, placeholders_comments)
@@ -771,7 +821,7 @@ def _detail_text_to_placholders(
 
         return _apply_placeholders(text, indices, base, "let".upper(), ptype)
 
-    if ptype == PlaceholderType.newif:
+    if ptype == PlaceholderType.newif_command:
         regex = r"(?<!\\)(\\newif)((\\[\w\@\*]*))*"
         if placeholders_comments is not None:
             is_comment = _is_placeholder(text, placeholders_comments)
@@ -860,12 +910,14 @@ def text_to_placeholders(
 
             $...$
 
-        (and other inline math environments) is replaced with
+        is replaced with
 
         .. code-block:: latex
 
             -BASE-INLINE-MATH-1-
 
+        Also looks for ``\(...\)`` and ``\begin{math}...\end{math}``.
+
     -   :py:class:`PlaceholderType.math`:
 
         .. code-block:: latex
@@ -874,14 +926,19 @@ def text_to_placeholders(
             ...
             \end{equation}
 
-        (and other math environments) is replaced with
+        is replaced with
 
         .. code-block:: latex
 
             -BASE-MATH-1-
 
+        Also looks for ``\[...\]``, ``\begin{equation*}...\end{equation*}``,
+        ``\begin{align}...\end{align}``, and ``\begin{align*}...\end{align*}``.
+
     -   :py:class:`PlaceholderType.math_line`:
 
+        A line of display mode math (see :py:class:`PlaceholderType.math`)
+
         .. code-block:: latex
 
             \begin{equation}
@@ -889,7 +946,7 @@ def text_to_placeholders(
             ...
             \end{equation}
 
-        (and other math environments) is replaced with
+        is replaced with
 
         .. code-block:: latex
 
@@ -938,6 +995,58 @@ def text_to_placeholders(
 
             -BASE-COMMAND-1-
 
+    -   :py:class:`PlaceholderType.command_like`:
+
+        .. code-block:: latex
+
+            {...}
+            \foo[...]{...}
+            {\foo[...]{...}}
+
+        is replaced with
+
+        .. code-block:: latex
+
+            -BASE-COMMAND-1-
+            -BASE-COMMAND-2-
+            -BASE-COMMAND-3-
+
+    -   :py:class:`PlaceholderType.curly_braced`:
+
+        .. code-block:: latex
+
+            {...}
+
+        is replaced with
+
+        .. code-block:: latex
+
+            -BASE-CURLY-BRACED-1-
+
+    -   :py:class:`PlaceholderType.let_command`:
+
+        .. code-block:: latex
+
+            \let\iffoo
+
+        is replaced with
+
+        .. code-block:: latex
+
+            -BASE-LET-1-
+
+    -   :py:class:`PlaceholderType.newif_command`:
+
+        .. code-block:: latex
+
+            \newif\iffoo
+
+        is replaced with
+
+        .. code-block:: latex
+
+            -BASE-NEWIF-1-
+
     :param text: Text.
     :param ptypes: List of placeholder types to replace
     :param base: Base string for placeholders
@@ -960,6 +1069,7 @@ def text_to_placeholders(
 def text_from_placeholders(
     text: str,
     placeholders: list[Placeholder],
+    keep_placeholders: bool = False,
 ) -> str:
     """
     Replace placeholders with original text.
@@ -968,6 +1078,7 @@ def text_from_placeholders(
 
     :param text: Text with placeholders.
     :param placeholders: List of placeholders.
+    :param keep_placeholders: If ``True``, the placeholders are kept (they are merely positioned).
     :return: Text with content of the placeholders.
     """
 
@@ -991,7 +1102,7 @@ def text_from_placeholders(
             if placeholder is None:
                 continue
             n = len(text)
-            text = placeholder.to_text(text, index + offset)
+            text = placeholder.to_text(text, index + offset, keep_placeholder=keep_placeholders)
             offset += len(text) - n
             if len(placeholders) == 0:
                 return text
@@ -999,7 +1110,7 @@ def text_from_placeholders(
     for key in placeholders:
         placeholder = placeholders[key]
         n = len(text)
-        text = placeholder.to_text(text)
+        text = placeholder.to_text(text, keep_placeholder=keep_placeholders)
 
     return text
 
@@ -1024,53 +1135,126 @@ def _lstrip_lines(text: str) -> str:
     return "\n".join([line.lstrip() for line in text.splitlines()])
 
 
-def _dedent(text: str, partial: list[PlaceholderType]) -> str:
-    """
-    Remove indentation.
+def _detail_align(lines: list[str], align: str, maxwidth: int = 150) -> str:
+    r"""
+    Align ``&`` and ``\\`` of all lines that contain those alignment characters.
 
     :param text: Text.
-    :param partial:
-        List of :py:class:`PlaceholderType` to dedent partially.
-        If the number of lines is less than 3, ``textwrap.dedent`` is applied to all lines.
-        Otherwise, ``textwrap.dedent`` is applied to ``lines[1:-1]``;
-        the first and last lines are stripped.
+    :param align: Alignment of columns (``"<"``, ``">"`` or ``"^"``).
+    :param maxwidth:
+        Alignment is applied only if the linewidth after alignment is smaller than ``maxwidth``.
+        Otherwise, spaces are added around ``&`` and before ``\\``.
 
-    :return: Formatted text.
+    :return: Aligned text.
     """
 
-    text, placholders = text_to_placeholders(text, partial, base="TEXDEDENT")
+    lines = [line.strip() for line in lines]
+    width = []
 
-    text = _lstrip_lines(text)
+    if len(lines) <= 1:
+        return lines
 
-    # keep common indentation table
-    # TODO: make more clever
-    for placeholder in placholders:
-        tmp = placeholder.content.splitlines()
-        if len(tmp) <= 2:
-            placeholder.content = "\n".join(textwrap.dedent(tmp))
+    for i, line in enumerate(lines):
+        # split at & and \\, and strip all spaces around
+        line = re.split(r"((?<!\\)&)", line)
+        line = line[:-1] + re.split(r"((?<!\\)\\\\)", line[-1])
+        line = list(filter(None, [i.strip() for i in line]))
+        if line[0] == "&":
+            line = [""] + line
+        lines[i] = line
+
+        # if line contains &: compute the width of each column
+        if "&" in line:
+            if len(width) == 0:
+                width = [len(col) for col in line]
+            else:
+                width += [len(col) for col in line[len(width) :]]
+                for j in range(len(line)):
+                    width[j] = max(width[j], len(line[j]))
+
+    # all lines start with &: remove leading spaces
+    if all([line[0] == "" for line in lines]):
+        lines = [line[1:] for line in lines]
+        width = width[1:]
+
+    if sum(width) < maxwidth:
+        fmt = " ".join("{" + str(i) + ":" + align + str(w) + "}" for i, w in enumerate(width))
+    else:
+        fmt = " ".join("{" + str(i) + "}" for i in range(len(width)))
+
+    for i, line in enumerate(lines):
+        if "&" in line:
+            lines[i] = fmt.format(*(line + [""] * (len(width) - len(line)))).rstrip()
         else:
-            placeholder.content = "\n".join(
-                [tmp[0].lstrip(), textwrap.dedent("\n".join(tmp[1:-1])), tmp[-1].lstrip()]
-            )
-        placeholder.space_front = "\n"
+            lines[i] = " ".join(line)
 
-    text = text_from_placeholders(text, placholders)
-
-    return text
+    return lines
 
 
-def _squashspaces(text: str, skip: list[PlaceholderType]) -> str:
+def _align(
+    placeholders: list[Placeholder],
+    placeholders_comments: list[Placeholder],
+    align: str = "<",
+) -> tuple[str, list[Placeholder]]:
     """
-    Squash spaces.
+    For all placeholders of environments:
+    -   Place ``\begin{...}[...]{...}`` and ``\\end{...}`` on own line.
+    -   Align ``&`` and ``\\`` of all lines that contain those alignment characters.
+
+    :warning: Assumes that each placeholder contains exactly one environment.
+    :warning: Assumes that all comments are replaced by placeholders.
+
+    :param placeholders: List of placeholders (changed in place).
+    :param placeholders_comments: List of placeholders that are comments.
+    """
+
+    for placeholder in placeholders:
+        content = placeholder.content.strip()
+        content = _begin_end_one_separate_line(content, placeholders_comments)
+        content = text_from_placeholders(content, placeholders_comments, keep_placeholders=True)
+        lines = content.strip().splitlines()
+        lines[1:-1] = _detail_align(lines[1:-1], align)
+        placeholder.content = "\n".join(lines)
+
+
+def align(
+    text: str,
+    environment: str,
+    align: str = "<",
+    base: str = "TEXINDENT-ALIGN",
+):
+    r"""
+    For all occurrences of an environment:
+    -   Place ``\begin{...}[...]{...}`` and ``\end{...}`` on own line.
+    -   Align ``&`` and ``\\`` of all lines that contain those alignment characters.
 
     :param text: Text.
-    :param skip: List of :py:class:`PlaceholderType` to skip.
+    :param environment: Name of the environment.
+    :param align: Alignment of columns (``"<"``, ``">"`` or ``"^"``).
+    :param base: Base for temporary placeholders.
     :return: Formatted text.
     """
 
-    text, placholders = text_to_placeholders(text, skip, base="TEXSQUASH")
-    text = re.sub(r"(\ +)", r" ", text)
-    text = text_from_placeholders(text, placholders)
+    text, placeholders_comments = text_to_placeholders(
+        text, [PlaceholderType.comment, PlaceholderType.inline_comment], base=base
+    )
+
+    indices = find_matching(
+        text,
+        r"\\begin{" + environment + r"}",
+        r"\\end{" + environment + r"}",
+        escape=False,
+        closing_match=1,
+        return_array=True,
+    )
+
+    text, placeholders = _apply_placeholders(
+        text, indices, base, environment.upper(), PlaceholderType.environment
+    )
+
+    _align(placeholders, placeholders_comments, align=align)
+    text = text_from_placeholders(text, placeholders)
+    text = text_from_placeholders(text, placeholders_comments)
     return text
 
 
@@ -1197,7 +1381,7 @@ def indent(text: str, indent: str = "    ") -> str:
 
     # comments: exclude from formatting
     text, placeholders_comment = text_to_placeholders(text, [PlaceholderType.comment])
-    text, placeholders_inline_comment = text_to_placeholders(text, [PlaceholderType.inline_comment])
+    text, placeholders_rcomment = text_to_placeholders(text, [PlaceholderType.inline_comment])
 
     # line comments: remove leading whitespace
     for placeholder in placeholders_comment:
@@ -1205,15 +1389,20 @@ def indent(text: str, indent: str = "    ") -> str:
         placeholder.space_front = re.sub(r"\ +", r" ", placeholder.space_front)
 
     # inline comments: remove duplicate leading spaces
-    for placeholder in placeholders_inline_comment:
+    for placeholder in placeholders_rcomment:
         placeholder.space_front = re.sub(r"\ +", r" ", placeholder.space_front)
 
-    placeholders_comments = placeholders_comment + placeholders_inline_comment
+    placeholders_comments = placeholders_comment + placeholders_rcomment
 
     # remove multiple newlines, duplicate spaces, and any leading whitespace
     text = re.sub(r"(\n\n+)", r"\n\n", text)
-    text = _dedent(text, partial=[PlaceholderType.tabular])
-    text = _squashspaces(text, skip=[PlaceholderType.tabular])
+    text = _lstrip_lines(text)
+    text = re.sub(r"(\ +)", r" ", text)
+
+    # format tables: aligns if possible
+    text, placeholders_table = text_to_placeholders(text, [PlaceholderType.tabular])
+    _align(placeholders_table, placeholders_comments)
+    text = text_from_placeholders(text, placeholders_table)
 
     # fold inline math
     text, placeholders_inline_math = text_to_placeholders(text, [PlaceholderType.inline_math])
@@ -1226,7 +1415,7 @@ def indent(text: str, indent: str = "    ") -> str:
 
     # put ``\begin{...}``/ ``\end{...}`` and ``\[`` / ``\]`` on a newline
     text, placeholders_let = text_to_placeholders(
-        text, [PlaceholderType.let, PlaceholderType.newif]
+        text, [PlaceholderType.let_command, PlaceholderType.newif_command]
     )
     text = _begin_end_one_separate_line(text, placeholders_comments)
     text = text_from_placeholders(text, placeholders_let)
@@ -1237,31 +1426,23 @@ def indent(text: str, indent: str = "    ") -> str:
     )
     text, placeholders_commands = text_to_placeholders(
         text,
-        [PlaceholderType.command, PlaceholderType.curly_braced],
+        [PlaceholderType.command_like],
         placeholders_comments=placeholders_comments,
     )
     text = _one_sentence_per_line(text)
     for placeholder in placeholders_commands:
-        placeholder.content = _format_command(
-            placeholder.content, placeholders_comments, placeholders_commands, placeholder.ptype
-        )
+        placeholder.content = _format_command(placeholder.content, placeholders_comments)
     text = text_from_placeholders(text, placeholders_ignore + placeholders_commands)
 
-    # place comment placeholders where they belong to do indentation
-    text = text_from_placeholders(text, placeholders_comments)
-    text, placeholders_comment = text_to_placeholders(
-        text, [PlaceholderType.comment, PlaceholderType.inline_comment]
+    # place placeholders where they belong to do indentation
+    # thereafter they should not be repositioned
+    text = text_from_placeholders(
+        text, placeholders_noindent + placeholders_comments, keep_placeholders=True
     )
-
-    # place noindent and verbatim placeholders where they belong to do indentation
-    text = text_from_placeholders(text, placeholders_noindent)
-    text, placeholders_noindent = text_to_placeholders(
-        text, [PlaceholderType.noindent_block, PlaceholderType.verbatim]
-    )
-
-    # fold math lines to simplify implementation
-    text, pl = text_to_placeholders(text, [PlaceholderType.math_line])
-    placeholders_comment += pl
+    for placeholder in placeholders_comments + placeholders_inline_math:
+        placeholder.space_front = None
+        placeholder.space_back = None
+    placeholders = placeholders_noindent + placeholders_comments + placeholders_inline_math
 
     # get line number of each character
     lineno = np.empty(len(text), dtype=int)
@@ -1273,6 +1454,7 @@ def indent(text: str, indent: str = "    ") -> str:
         lineno[i] = line
         i += 1
     lineno[i:] = line + 1
+    lineno = np.append(lineno, line + 2)
 
     # initialize indentation level
     indent_level = np.zeros(lineno[-1] + 1, dtype=int)
@@ -1305,7 +1487,7 @@ def indent(text: str, indent: str = "    ") -> str:
         indent_level[lineno[indices[i, 0]] + 1 : lineno[indices[i, 1]]] += 1
 
     # add indentation to all command options ``[`` and ``]`` containing at least one ``\n``
-    commands = find_command(text, is_comment=_is_placeholder(text, placeholders_comment))
+    commands = find_command(text, is_comment=_is_placeholder(text, placeholders_comments))
     indices = []
     for command in commands:
         if len(command) < 2:
@@ -1332,13 +1514,12 @@ def indent(text: str, indent: str = "    ") -> str:
         indent_level[lineno[match.span(2)[0]]] -= 1
 
     # apply indentation
-    text = text_from_placeholders(text, placeholders_comment)
     text = text.splitlines()
     for i in range(len(text)):
         text[i] = indent_level[i] * indent + text[i]
     text = "\n".join(text)
 
-    text = text_from_placeholders(text, placeholders_inline_math + placeholders_noindent)
+    text = text_from_placeholders(text, placeholders)
     return _rstrip_lines(text)
 
 
@@ -1355,7 +1536,7 @@ def _detail_one_sentence_per_line(text: str) -> str:
     text = re.split(r"(?<=[\.\!\?])\s+", text)
 
     for i in range(len(text)):
-        text[i] = re.sub("(\n[\\ \t]*)([\\w\\$\\(\\[\\`])", r" \2", text[i])
+        text[i] = re.sub(r"([^\n]+)(\n[\ \t]*)([\w\$\(\[\`])", r"\1 \3", text[i])
 
     return "\n".join(text)
 
@@ -1388,6 +1569,9 @@ def _one_sentence_per_line(
     # \end{...}
     skip += [i.span() for i in re.finditer(r"(?<!\\)(\\)(end\{\w*\}\s*)", text)]
 
+    # \\
+    skip += [i.span() for i in re.finditer(r"(\\\\)", text)]
+
     # multiple newlines
     skip += [i.span() for i in re.finditer(r"(\n\n+)", text)]
 
@@ -1396,6 +1580,7 @@ def _one_sentence_per_line(
     else:
         skip = np.array(skip)
         skip = _filter_nested(skip[skip[:, 0].argsort()])
+        # merge consecutive blocks
         keep = np.ones(skip.shape[0], dtype=bool)
         for i in range(skip.shape[0] - 1):
             if skip[i, 1] == skip[i + 1, 0]:
@@ -1422,8 +1607,6 @@ def _one_sentence_per_line(
 def _format_command(
     text: str,
     placeholders_comments: list[Placeholder],
-    placeholders_commands: list[Placeholder],
-    ptype: PlaceholderType = PlaceholderType.command,
     level: int = 0,
 ) -> str:
     """
@@ -1445,13 +1628,17 @@ def _format_command(
 
     :param text: Text.
     :param placeholders_comments: List of placeholders for comments.
-    :param level: Level of nestedness, used to define unique placeholder names.
+    :param ptype: Placeholder type.
+    :param level: Level of nested-ness, used to define unique placeholder names.
     :return: Formatted text.
     """
     if not re.match(r".*\n.*", text):
         return text
 
-    if ptype == PlaceholderType.command:
+    if text[0] == "{":
+        parts = ["", text, ""]
+
+    else:
         is_comment = _is_placeholder(text, placeholders_comments)
         commands = find_command(text, is_comment=is_comment)
         commands = [i for i in commands if len(i) > 1]
@@ -1471,13 +1658,6 @@ def _format_command(
             o, c = braces[i]
             parts += [text[o:c], text[c : braces[i + 1][0]]]
 
-    elif ptype == PlaceholderType.curly_braced:
-        parts = ["", text, ""]
-    else:
-        raise ValueError(f"Unknown placeholder type {ptype}")
-
-    search_placeholder = list(set(list({i.search_placeholder for i in placeholders_commands})))
-
     for i, part in enumerate(parts):
         if i % 2 == 1:
             if not re.match(r".*\n.*", part):
@@ -1485,34 +1665,15 @@ def _format_command(
             body = part[1:-1].strip()
             body, placeholders_cmd = text_to_placeholders(
                 body,
-                [PlaceholderType.command, PlaceholderType.curly_braced],
+                [PlaceholderType.command_like],
                 f"TEXONEPERLINE-L{level}",
             )
             body = _one_sentence_per_line(body, [], command=True)
 
-            # make sure that the newline is kept
-            # TODO this is a hack, find a better solution
-            for search in search_placeholder:
-                for match in re.finditer(search, body):
-                    if match.span()[0] == 0:
-                        pl = match.group()
-                        for ipl in range(len(placeholders_commands)):
-                            if placeholders_commands[ipl].placeholder == pl:
-                                placeholders_commands[ipl].space_front = None
-                                break
-                    if match.span()[1] == len(body):
-                        pl = match.group()
-                        for ipl in range(len(placeholders_commands)):
-                            if placeholders_commands[ipl].placeholder == pl:
-                                placeholders_commands[ipl].space_back = None
-                                break
-
             for placeholder in placeholders_cmd:
                 placeholder.content = _format_command(
                     placeholder.content,
                     placeholders_comments,
-                    placeholders_cmd,
-                    placeholder.ptype,
                     level + 1,
                 )
             body = text_from_placeholders(body, placeholders_cmd)
